# !/usr/bin/env python
#  -*- coding: UTF-8 -*-

__version__ = "0.1"  # LATEST? =>  https://pypi.org/project/XXXXX/
__copyright__ = "CopyRight (C) 2021 by Donald Duck"
__license__ = "MIT"
__author__ = "Donald Duck"
__author_email__ = ""

USAGE = "lookmlhelper [options]"
VERSION = "v" + __version__
AGENT = "%s/%s" % (__name__, __version__)
